// implementation of static repository

use super::domain::{AdminModelConfig, AdminRepository, RepositoryInfo, RepositoryList};
use super::state::AdminRegistry;
use serde_json::{json, Value};

struct MyStaticRepository {
    content: Vec<Value>,
}

impl MyStaticRepository {
    pub fn new() -> Self {
        MyStaticRepository {
            content: vec![
                json!({"id": 1, "name": "Strange", "age": 150, "level": "master" }),
                json!({"id": 2, "name": "Adam", "age": 12, "powers": 8}),
                json!({"id": 3, "name": "Tony", "age": 42, "powers": 0}),
                json!({"id": 4, "name": "Rex", "age": 72}),
                json!({"id": 5, "name": "Justin", "age": 46}),
                json!({"id": 6, "name": "Reacher", "age": 39, "level": "adept", "powers": 0}),
                json!({"id": 7, "name": "Arnold", "age": 64}),
            ],
        }
    }
}

impl AdminRepository for MyStaticRepository {
    fn get_item(&self, id: usize) -> Option<Value> {
        self.content.get(id).cloned()
    }

    fn get_list(&self) -> RepositoryList {
        RepositoryList::List {
            values: self.content.clone(),
        }
    }

    fn get_repo_info(&self) -> RepositoryInfo {
        RepositoryInfo::new("My Static Repository", "id")
            .display_list(&["id", "name", "age", "level"])
    }
}

pub fn register_example(registry: &mut AdminRegistry) {
    let app_key = registry.register_app("Example App");
    let repo = MyStaticRepository::new();
    let model_config = AdminModelConfig {
        app_key: app_key,
        name: "ExampleModel".to_owned(),
    };
    let model_result = registry.register_model(model_config, repo);
    match model_result {
        Err(err) => panic!("{}", err),
        _ => (),
    }
}
